! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!+
! Module: pnspectra_mod
!
! Authors:  Brendan Perry - NASA/GSFC, brendan.h.perry@nasa.gov (f95 rewrite/SAS compliance)
!           Kip Kuntz - NASA/GSFC/JHU (original algorithm)
!           Steven L. Snowden - NASA/GSFC (original algorithm)
! 
! Description: Former Extended Source Analysis Software subtask
!
! Comments:
!
! $Id: pnspectra_mod.f90,v 1.3 2009/02/26 16:39:45 aibarra Exp $
!-
!  Version 0.11 - complete rewrite of sas task pn-spectra.pl into f90 by BP
!
! This task creates intermediary files for creation of model particle background
! spectra and images by pnback
!
!  usage: pnspectra + 
!  eventfile -- event list from espfilt
!  ootevtfile -- OOT event list from espfilt
!  cornerfile -- corner event list from espfilt
!  ootcornfile -- OOT corner event list from espfilt
!  imagefile -- Entire FOV region observation image.
!  ootimgfile -- Entire FOV region observation image.
!  expmap -- Exposure map in DET coords
!  spmask -- Mask for calculating soft proton scaling (FOV image for all events)
!  specfile -- FOV spectrum full band 
!  rmffile -- Response matrix file name (will construct name if none given).
!  arffile -- Auxillary response matrix file name (will construct name if none given).
!  keepinterfiles -- Keep intermediate files created (F to delete)
!  withregion -- Input a text region file?
!  regionfile -- Text file with additional region information.
!  withmasksky -- Input a SKY coord mask FITS file?
!  masksky -- SKY coord mask FITS file name.
!  withmaskdet -- Input a DET coord mask FITS file?
!  maskdet -- DET coord mask FITS file name.
!  elow -- Band low energy limit (elow).
!  ehigh -- Band high energy limit (ehigh).
!  quads -- PN quadrant (1-4)
!  pattern - evselect PATTERN (1 or 4)
!
!  The event file should be cleaned of bad time intervals,
!  bad pixels, and the like, but MUST include the corner pixels.
!
!  SAS must be set up and running in the window
!  OLD: 
!  obj - observation data from the selected region
!  oc - observation data from the corners
!  ff - filter wheel closed from the selected region
!  fc - filter wheel closed from the corners
!
module pnspectra_mod

  use dal
  use cal
  use param
  use dataFilter
  use errorHandling
  use sasdal
  use gofio_mod
  use gofutils_mod

  implicit none

  private

  character(len=*), parameter :: &
       eventpar    = "eventfile", &    ! input EPN event list from espfilt
       ootpar      = "ootevtfile", &   ! input EPN event list from evselect
       cornerpar   = "cornerfile", &   ! input EPN event list from espfilt
       cornootpar  = "ootcornfile", &  ! input EPN event list from espfilt
       imagepar    = "imagefile", &    ! input EPN image from espfilt
       ootimpar    = "ootimgfile", &   ! input EPN image from espfilt
       expmappar   = "expmap", &       ! input EPN exposure map
       spmappar    = "spmask", &       ! input EPN SP map
       specpar     = "specfile", &     ! input EPN FOV spectrum
       maskpar     = "mask", &         ! input EPN SP map
       withregpar  = "withregion", &   ! Apply region file?
       regpar      = "regionfile", &   ! text region file
       withskypar  = "withmasksky", &  ! Apply background region in sky coords?
       skypar      = "masksky", &      ! input EPN bkg region file
       withdetpar  = "withmaskdet", &  ! Apply background region in det coords?
       detpar      = "maskdet", &      ! input EPN bkg region file
       rmfpar      = "rmffile", &      ! response matrix file
       arfpar      = "arffile",&       ! auxilliary response file
       quadspar    = "quads", &        ! user-selected quads
       elowpar     = "elow", &         ! Low energy limit for the bands (eV)
       ehighpar    = "ehigh", &        ! High energy limit for the bands (eV)
       pattpar     = "pattern", &      ! input EPN SP map
       keeppar     = "keepinterfiles"  ! keep (delete) intermediary files?

  public pnspectra

contains

  !---------------------------------------------------------------------
  subroutine pnspectra
    !---------------------------------------------------------------------

    type(DataSetT) :: inEVLI_S, inSKY_S, inDET_S, inOOT_S, temp_S
    type(ArrayT)   :: inEVLI_A
    type(TableT)   :: inEVLI_T
    type(CalInfo) :: fwcFileInfo
    !
    character(len=1) :: schip, cband
    character(len=2) :: instE, inst, pattern
    character(len=5) :: expidE, mode
    character(len=6) :: celow, cehigh, filterE
    character(len=10) :: obsidE
    character(len=18) :: prefix, type
    character(len=25) :: smodeE
    character(len=40) :: maskitdet, maskitsky
    character(len=60) :: quaddefa
    character(len=256) :: inEVLI_F, inCORN_F, inIMG_F, inEXPMAP_F, inSPMASK_F, &
         inREG_F, inSKY_F, inDET_F, inSPEC_F, inRMF_F, inARF_F, inMASK_F, inFWC_F, &
         inEXPMASK_F, inSPMAP_F, inBAND_F, quadIMG_F, quadSPEC_F, quadMASK_F, &
         inFWCOOT_F, inOOT_F, inCORNOOT_F, inIMGOOT_F, quadEXPIMG_F, &
         inEVLIOOT_F, maskIMG_F, atthk_F
    character(len=256) :: ccddef
    character(len=1024) :: command, corndef, fulldef, fovdef, ruffovdef, &
         quaddef(4), ccfdir
    logical :: i_ok, warndie, pn, keep, dobands, go, quads(4)
    logical :: withdet, withsky, withreg
    integer :: frmtimeE, cnts, cntsoot
    integer :: typeF, nRowsE, revolutE, ff, cc
    integer :: elow, ehigh, ehighE, elowE, ilo, ihi, cflim
    integer :: i, j, k, n
    integer :: status = 0
    real :: hard, hardsig, livetime
    real :: hige, lowe, higeoot, loweoot, rate, scale
    real :: backE, areaE, expoE, expo, expooot
    !
    inSKY_F    = ""
    inDET_F    = ""
    inREG_F    = ""
    inSPMASK_F = ""
    inMASK_F   = ""
    inSPEC_F   = ""
    inEXPMAP_F = ""
    inOOT_F    = ""
    maskitsky  = ""
    maskitdet  = ""
    !    
    ! read all parameters
    ! eventlist comes from espfilt
    ! OLD: pnS001-clean.fits NEW: pnS001-fovevc.fits from espfilt
    inEVLI_F    = stringParameter(eventpar)  ! pnS001-fovevc.fits
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(inEVLI_F),inEVLI_S,1,status,warndie)
    !
    ! Get critical header keywords from input eventlist
    !
    status = 0
    warndie = .TRUE. ! if not found, unreadable, die
    typeF = 1 ! 1 = EVLI, 2 = SPEC, 3 = IMG, most of these are dummies
    call read_header_info(typeF,inEVLI_S,inEVLI_T,inEVLI_A,instE,obsidE,&
         expidE,smodeE,frmtimeE,filterE,revolutE,backE,areaE,expoE,elowE,&
         ehighE,nRowsE,warndie)
    ! set prefix for filename construction usage
    inst = "EPN"
    if (instE .eq. 'PN') then
       prefix = "pn" // trim(adjustl(expidE)) ! mos1S001
       call CAL_setState(instrument=EPN)
    else
       call error("notPN","Input EVLI must be PN")
    endif
    call CAL_fwcInfo(fwcFileInfo)
    inFWC_F = fwcFileInfo%fileName
    write(errstr,*) 'FWC CCF filename : ', trim(adjustl(fwcFileInfo%fileName))
    call message(AppMsg,Always) ! change to Verbose???!!!
    prefix = "pn" // trim(adjustl(expidE)) ! pnS001
    ! call get_environment_variable("SAS_CCFPATH",ccfdir)
    ! inFWC_F = trim(adjustl(ccfdir)) // "/EPN_FWC_0003.CCF" ! need cal call to get file
    !
    ! set the OOT scale and mode
    !
    SELECT CASE (trim(adjustl(smodeE)))
    CASE ('PrimeFullWindowExtended')
       if (frmtimeE .lt. 210) then
          scale = 0.0232
          smodeE = 'PrimeFullWindowExtended3'
          mode = 'eff3'
       else
          scale = 0.0163
          smodeE = 'PrimeFullWindowExtended5'
          mode = 'eff5'
       endif
    CASE ('PrimeFullWindow')
       mode = 'ff'
       scale = 0.063
    CASE default ! check this ???!!!
       call error('badSMODE',&
            "SUBMODE must be PrimeFullWindowExtended or PrimeFullWindow")
    END select
    ! now get the rest of the input parameters
    ! this cleaned fov OOT event list comes from espfilt
    inEVLIOOT_F   = stringParameter(ootpar) ! pnS001-fovevcoot.fits
    if (trim(adjustl(inEVLIOOT_F)) .eq. 'default') then
       inEVLIOOT_F = trim(adjustl(prefix)) // "-fovevcoot.fits"
    endif
    ! this cleaned corner OOT event list comes from espfilt
    inCORN_F   = stringParameter(cornootpar) ! pnS001-corevc.fits
    if (trim(adjustl(inCORN_F)) .eq. 'default') then
       inCORN_F = trim(adjustl(prefix)) // "-corevc.fits"
    endif
    ! this cleaned corner OOT event list comes from espfilt
    inCORNOOT_F   = stringParameter(cornootpar) ! pnS001-corevc.fits
    if (trim(adjustl(inCORNOOT_F)) .eq. 'default') then
       inCORNOOT_F = trim(adjustl(prefix)) // "-corevcoot.fits"
    endif
    ! this cleaned fov image comes from pnspectra, cheese, or espfilt !!!
    inIMG_F    = stringParameter(imagepar)  ! pnS001-fovimt.fits fovimt???!!!
    if (trim(adjustl(inIMG_F)) .eq. 'default') then
       ! fovim or fovimc or fovimt/s/ t and c should be ~same, no???!!!
       inIMG_F = trim(adjustl(prefix)) // "-fovimt.fits"
    endif
    ! 
    inIMGOOT_F = stringParameter(ootimpar)  ! pnS001-fovimcoot.fits
    if (trim(adjustl(inIMGOOT_F)) .eq. 'default') then
       ! fovim or fovimc or fovimt/s/ t and c should be ~same, no???!!!
       inIMGOOT_F = trim(adjustl(prefix)) // "-fovimtcoot.fits"
    endif
    ! this SP scaling file is created only by pnspectra
    inSPMASK_F = stringParameter(spmappar) ! pnS001-fovspdet.fits
    if (trim(adjustl(inSPMASK_F)) .eq. 'default') then
       ! fovspdet t? h? s???
       inSPMASK_F = trim(adjustl(prefix)) // "-fovimspdet.fits"
    endif
    ! this mask band region file is created only by pnspectra
    inMASK_F = stringParameter(maskpar) ! pnS001-maskimt.fits
    if (trim(adjustl(inMASK_F)) .eq. 'default') then
       ! t? h? s???
       inMASK_F = trim(adjustl(prefix)) // "-maskimt.fits"
    endif
    ! this mask band region file is created only by pnspectra
    inSPEC_F = stringParameter(specpar) ! pnS001-fovt.pi
    if (trim(adjustl(inSPEC_F)) .eq. 'default') then
       ! t? h? s???
       inSPEC_F = trim(adjustl(prefix)) // "-fovt.pi"
    endif
    ! this exposure map is created cheese or pnspectra
    inEXPMAP_F = stringParameter(expmappar) ! pnS001-expimt.fits
    if (trim(adjustl(inEXPMAP_F)) .eq. 'default') then
       !  t? h? s??? can use fovimtexp file???
       inEXPMAP_F = trim(adjustl(prefix)) // "-expimt.fits"
    endif
    ! this file comes from cheese (apply a sky coord region mask)
    withsky    = booleanParameter(withskypar)
    if (withsky) then
       inSKY_F = stringParameter(skypar)   ! pnS001-bkregtsky.fits
       ! can trigger withsky but if no name given, creates a default name
       ! "bkg region, total band, sky coords" = bkregtsky
       if (trim(adjustl(inSKY_F)) .eq. 'default') then
          ! gonogo???
          inSKY_F = trim(adjustl(prefix)) // "-bkregtsky.fits"
       endif
    end if
    ! this file comes from cheese (apply a sky coord region mask)
    withdet    = booleanParameter(withdetpar)
    if (withdet) then
       inDET_F = stringParameter(detpar)   ! pnS001-bkregtdet.fits
       ! can trigger withdet but if no name given, creates a default name
       ! "bkg region, total band, det coords" = bkregtdet
       if (trim(adjustl(inDET_F)) .eq. 'default') then
          ! gonogo???
          inDET_F = trim(adjustl(prefix)) // "-bkregtdet.fits"
       endif
    end if
    withreg    = booleanParameter(withregpar)
    if (withreg) then
       inREG_F = stringParameter(regpar)   ! reg.txt
    end if
    do i=1,4
       quads(i) = booleanParameter(quadspar,i-1) ! T T T T
    enddo
    ! user-specified, but if still 'default', generate rmf as usual 
    inRMF_F = stringParameter(rmfpar)
    inARF_F = stringParameter(arfpar)
    if (trim(adjustl(inRMF_F)) .eq. 'default') then
       inRMF_F = trim(adjustl(prefix)) // ".rmf"
    end if
    if (trim(adjustl(inARF_F)) .eq. 'default') then
       inARF_F = trim(adjustl(prefix)) // ".arf"
    endif
    !
    pattern = ""
    pattern = stringParameter(pattpar)  ! evselect PATTERN
    pattern = trim(adjustl(pattern)) // ")"
    keep = booleanParameter(keeppar) ! T or F, delete intermediary files
    !
    celow = ''
    cehigh = ''
    elow  = intParameter(elowpar)      ! 0-11999, constrained in params
    ehigh = intParameter(ehighpar)     ! 1-12000, constrained in params
    !
    dobands = .TRUE.
    if ((elow .eq. 0) .and. (ehigh .eq. 0)) then ! is this EVER true???
       dobands = .FALSE.
    end if
    write(celow,'(I5)') elow
    write(cehigh,'(I5)') ehigh
    !    if (verbosity(AppMsg) > Verbose) then
    call printParams(inEVLI_F,inEVLIOOT_F,inCORN_F,inCORNOOT_F,inIMG_F,inIMGOOT_F,&
         inSPMASK_F,inEXPMAP_F,inMASK_F,withsky,inSKY_F,withdet,inDET_F,withreg,&
         inREG_F,inRMF_F,inARF_F,quads,elow,ehigh,dobands,keep)
    !    endif
    !
    ! Definitions
    !   ccddef is the selection criteria for the active CCDs, i.e., 
    !        those not excluded by the quad# parameter
    !   corndef is a definition of the corner pixels - it excludes
    !        the open FOV of the detector
    !   quaddef is a definition of the location of each quadrant
    !        in DETX,DETY space
    !   fulldef is a definition of box containing the full CCD area
    !        and provides no additional restrictions, it just allows
    !        backscale to be happy when calculating the BACKSCAL
    !        keyword for the corner spectra.
    !   ruffovdef is the circle restricting the FOV to the area
    !        open to the sky.
    !   psdef is the point-source exclusion as selected by the input
    !        parameter. check this???!!! seems removed in perl
    call defineQUADreg (quads,ccddef)
    call defineexpr (corndef,quaddef,fulldef,ruffovdef)
    !
    !  Add the source exclusion file if desired
    !
    if (withsky) then ! if withsky and OLD:  pn-bkg_region-sky.fits exists
       if (trim(adjustl(inSKY_F)) .eq. 'default') then
          inSKY_F = trim(adjustl(prefix)) // "bkregtsky.fits" ! auto construct
       end if
       status = 0 !                    NEW:  pnS001-bkregtsky.fits (or h or s) 
       warndie = .FALSE.
       call check_open_set(trim(inSKY_F),inSKY_S,1,status,warndie)
       if (status .eq. 0) then
          write(errstr,*) "SKY region file exists. Proceed with source exclusion."
          call message(AppMsg,Always)
          maskitsky = "&&region(" // trim(adjustl(inDET_F)) // ")"
       else
          write(errstr,*) "SKY region file does NOT exists. Will not source exclude."
          call message(AppMsg,Always)
       end if
    endif
    if (withdet) then ! if withsky and OLD:  pn-bkg_region-det.fits exists
       if (trim(adjustl(inDET_F)) .eq. 'default') then
          inDET_F = trim(adjustl(prefix)) // "bkregtdet.fits" ! auto construct
       end if
       status = 0
       warndie = .FALSE.
       call check_open_set(trim(inDET_F),inDET_S,1,status,warndie)
       if (status .eq. 0) then
          write(errstr,*) "DET region file exists. Proceed with source exclusion."
          call message(AppMsg,Always)
          maskitdet = "&&region(" // trim(adjustl(inDET_F)) // ")"
       else
          write(errstr,*) "DET region file does NOT exists. Will not source exclude."
          call message(AppMsg,Always)
       end if
    endif
    !
    ! open and read region file, update withreg if can't read/find regionfile
    ! Get the definition of the object region
    fovdef = ""
    call read_region(inREG_F,fovdef,withreg)
    ! Create an attitude file
    go = .FALSE.
    atthk_F = "atthk.fits"
    call gonogo(atthk_F,go)
    if (go) then
       write(errstr,*) "Making atthk file..."
       call message(AppMsg,Verbose)
       command = "atthkgen atthkset=" // trim(adjustl(atthk_F)) // " timestep=1"
       call system(trim(command))
    else
       write(errstr,*) "atthk.fits file exists will not rerun atthkgen."
       call message(AppMsg,Always)
    end if
    !
    ! If needed, extract the corner region from observation data into event file
    ! Extract the corner region from observation data (and OOT) into event file
    ! docorner does both the corner and oot corner selections
    ! OLD: pnS001-corn.fits NEW: pnS001-corevc.fits
    ! OLD: pnS001-corn-oot.fits NEW: pnS001-corevcoot.fits
    call docorner(inEVLI_F,inEVLIOOT_F,inCORN_F,inCORNOOT_F,pattern,ccddef,corndef)
    !
    ! Extract the entire FOV region from observation data into an image
    ! 
    ! OLD pnS001-clean + pnS001-bkg_region-sky.fits  = pnS001-obj-im.fits
    ! NEW pnS001-fovevc.fits + pnS001-bkreghsky.fits = pnS001-fovimt.fits (or fovim???!!!)
    call makeImFile(inEVLI_F,inIMG_F,maskitsky,pattern,ccddef,fulldef,ruffovdef)
    ! Extract the entire FOV region from observation data into an image - OOT data
    ! OLD pnS001-clean-oot + pnS001-bkg_region-sky.fits  = pnS001-obj-im-oot.fits
    ! NEW pnS001-fovevcoot.fits + pnS001-bkreghsky.fits = pnS001-fovimtoot.fits (or fovim???!!!)
    call makeImFile(inEVLIOOT_F,inIMGOOT_F,maskitsky,pattern,ccddef,fulldef,ruffovdef)
    ! Now make an exposure map for the FOV image
    ! OLD pnS001-clean.fits + pnS001-obj-im.fits = pnS001-exp-im.fits
    ! NEW pnS001-fovevc.fits + pnS001-fovimt.fits = pnS001-expimt.fits
    ! inEXPMAP_F can be user selected, defaults to 
    call gonogo(inEXPMAP_F,go) ! exposure map
    if (go) then
       write(errstr,*) "Making exposure map for the FOV image..."
       call message(AppMsg,Verbose)
       command = "eexpmap attitudeset=atthk.fits eventset=" // &
            trim(adjustl(inEVLI_F)) // ":EVENTS " // &
            "expimageset=" // trim(adjustl(inEXPMAP_F)) // &
            " imageset=" // trim(adjustl(inIMG_F)) // &
            " pimax=8000 pimin=300 withdetcoords=no"
       call system(trim(command))
    else
       write(errstr,*) "FOV exposure map already exists will not rerun eexpmap here."
       call message(AppMsg,Always)
    end if
    ! Make a mask 
    ! OLD: pnS001-obj-im.fits -> pnS001-mask-im.fits
    ! NEW: pnS001-fovim(c?h/s/t?).fits -> pnS001-maskim(h/s/t?).fits
    call gonogo(inMASK_F,go) ! mask file pnS001-fovmaskt.fits from cheese???!!!
    if (go) then
       call run_emask(inMASK_F,inIMG_F)
    else
       write(errstr,*) "FOV mask from FOV image exists, will not rerun emask here."
       call message(AppMsg,Verbose)
    end if
    !
    ! Extract the selected region from observation data into an image
    ! Do it again but use all events.  This will be a mask for calculating
    ! the SP scaling
    ! 
    ! OLD: pnS001-clean.fits + pnS001-bkg_region-sky.fits = pnS001-obj-im-sp-det.fits
    ! NEW: pnS001-fovevc.fits + pnS001-bkregtsky.fits = pnS001-fovspdet.fits
    ! gonogo for spmask???!!! 
    call makeSPfile(inEVLI_F,inSPMASK_F,maskitdet,fulldef,ccddef,quaddefa,fovdef,&
         ruffovdef,celow,cehigh,pattern)
    !
    if (dobands) then ! OLD: ehigh > 0, the non-default (fovimc???!!!)
       ! Extract the selected region and band from observation data into an 
       ! image in detector coords (DET)
       ! OLD: pnS001-clean.fits + pnS001-bkg_region-det.fits = pnS001-obj-im-350-1100.fits
       ! NEW: pnS001-fovevc.fits + pnS001-bkreghdet.fits = pnS001-fovimc-350-1100-det.fits 
       inBAND_F = trim(adjustl(prefix)) // "-fovimdet-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       call gonogo(inBAND_F,go) 
       ! gonogo here???!!!
       call makeBandImageDet (inEVLI_F,inBAND_F,maskitdet,fulldef,ccddef,quaddefa,fovdef,&
            ruffovdef,celow,cehigh,pattern)
       ! Make an exposure map in detector coords for the band
       ! OLD: pnS001-exp-im-det-350-1100.fits
       ! NEW: pnS001-expimdet-350-1100.fits
       ! reuse variable inBAND_F
       inBAND_F = trim(adjustl(prefix)) // "-expimdet-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       call gonogo(inBAND_F,go) ! exposure map
       if (go) then
          write(errstr,*) "Making exposure map in DET coords for band..."
          call message(AppMsg,Verbose)
          command = "eexpmap attitudeset=atthk.fits eventset=" // &
               trim(adjustl(inEVLI_F)) // ":EVENTS " // &
               "expimageset=" // trim(adjustl(inEXPMAP_F)) // &
               " imageset=" // trim(adjustl(inIMG_F)) // &
               " pimax=" // trim(adjustl(cehigh)) // "  pimin=" // &
               trim(adjustl(celow)) // " withdetcoords=no"
          call system(trim(command))
       else
          write(errstr,*) "Exposure map already exists will not rerun eexpmap here."
          call message(AppMsg,Always)
       end if
       !
       ! Make a mask for the band
       ! OLD: pnS001-mask-im-det-350-1100.fits
       ! NEW: pnS001-maskimdet-350-1100.fits
       ! reuse inBAND_F
       inBAND_F = trim(adjustl(prefix)) // "-maskimdet-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       call gonogo(inBAND_F,go) ! exposure map
       if (go) then
          write(errstr,*) "Making mask in DET coords for band..."
          call message(AppMsg,Verbose)
          call run_emask(inBAND_F,inIMG_F)
       else
          write(errstr,*) "Mask for region and band exists, will not rerun emask here."
          call message(AppMsg,Always)
       end if

       ! Make another image in detector coords from OOT events
       ! OLD: pnS001-obj-im-det-350-1100-oot.fits"
       ! NEW: pnS001-fovimcootdet-350-1100.fits"
       ! reuse inBAND_F
       inBAND_F = trim(adjustl(prefix)) // "-fovimdetoot-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       call gonogo(inBAND_F,go) ! 
       if (go) then
          write(errstr,*) "Making OOT image in DET coords for band..."
          call message(AppMsg,Verbose)
          call makeBandImageDet (inEVLI_F,inBAND_F,maskitdet,fulldef,ccddef,quaddefa,fovdef,&
               ruffovdef,celow,cehigh,pattern)
       else
          write(errstr,*) "OOT image in band exists, will not rerun evselect"
          call message(AppMsg,Always)
       end if

    end if
    !
    !  Extract the selected FOV region from observation data into spectral file
    !  then run backscale, generate response
    ! OLD: pnS001-clean.fits  + pnS001-bkg_region-det.fits = pnS001-obj.pi
    ! NEW: pnS001-fovevc.fits + pnS001-bkreghsky.fits      = pnS001-fovt.pi
    if (trim(adjustl(inSPEC_F)) .eq. 'default') then
       inSPEC_F = trim(adjustl(prefix)) // "-fovt.pi"
    endif
    call gonogo(inSPEC_F,go) ! NEW: pnS001-fovt.pi OLD: pnS001-obj.pi 
    if (go) then
       write(errstr,*) "Making FOV region spectral file..."
       call message(AppMsg,Verbose)
       ! it doesn't matter if withreg or withmaskdet
       ! maskitdet and fovdef are null strings unless actually specified 
       command = "evselect table=" // trim(adjustl(inEVLI_F)) // &
            ":EVENTS withfilteredset=yes " // & 
            "expression='(PATTERN<=" // trim(adjustl(pattern)) // &
            "&&(FLAG == 0)" // trim(adjustl(ccddef)) // trim(adjustl(fulldef)) // &
            trim(adjustl(fovdef)) // trim(adjustl(ruffovdef)) // &
            trim(adjustl(maskitdet)) // "' filtertype=expression keepfilteroutput=no " // &
            "updateexposure=yes "// "filterexposure=yes withspectrumset=yes " // &
            "spectrumset=" // trim(adjustl(inSPEC_F)) // " energycolumn=PI " // & 
            "spectralbinsize=5 withspecranges=yes specchannelmin=0 " // &
            "specchannelmax=20479"
       call system(trim(command))
    else
       write(errstr,*) "Spec file for FOV exists, will not rerun evselect here."
       call message(AppMsg,Always)
    end if
    ! run backscale just in case even if exists???
    write(errstr,*) "Running backscale on FOV spectrum..."
       call message(AppMsg,Always)
    command = "backscale spectrumset=" // trim(adjustl(inSPEC_F)) // &
         " badpixlocation=" // trim(adjustl(inEVLI_F)) // " withbadpixcorr=yes"
    call system(trim(adjustl(command)))
    !
    !  Extract OOT spectrum
    !  then run backscale, generate response
    ! OLD: pnS001-clean.fits + pnS001-bkg_region-det.fits = pnS001-obj.pi
    ! NEW: pnS001-fovevc.fits +  pnS001-bkreghsky.fits = pnS001-fovh.pi !!!    
    inSPEC_F = trim(adjustl(prefix)) // "-fovtoot.pi"
    call gonogo(inSPEC_F,go) ! pnS001-fov.pi OLD: pnS001-obj.pi (default, now user-specified)
    if (go) then
       write(errstr,*) "Making OOT region spectral file..."
       call message(AppMsg,Verbose)
       ! it doesn't matter if withreg or withmaskdet
       ! maskitdet and fovdef are null strings unless actually specified 
       command = "evselect table=" // trim(adjustl(inEVLI_F)) // &
            ":EVENTS withfilteredset=yes " // & 
            "expression='(PATTERN<=" // trim(adjustl(pattern)) // &
            "&&(FLAG == 0)" // trim(adjustl(ccddef)) // trim(adjustl(fulldef)) // &
            trim(adjustl(fovdef)) // trim(adjustl(ruffovdef)) // &
            trim(adjustl(maskitdet)) //  "' filtertype=expression keepfilteroutput=no " // &
            "updateexposure=yes "// "filterexposure=yes withspectrumset=yes " // &
            "spectrumset=" // trim(adjustl(inSPEC_F)) // " energycolumn=PI " // & 
            "spectralbinsize=5 withspecranges=yes specchannelmin=0 " // &
            "specchannelmax=20479"
       call system(trim(command))
     else
       write(errstr,*) "Spec file for OOT exists, will not rerun evselect here."
       call message(AppMsg,Always)
    end if
    write(errstr,*) "Running backscale on FOV spectrum..."
       call message(AppMsg,Verbose)
    command = "backscale spectrumset=" // trim(adjustl(inSPEC_F)) // &
         " badpixlocation=" // trim(adjustl(inEVLI_F)) // " withbadpixcorr=yes"
    call system(trim(adjustl(command)))
    !
    !  Generate the RMF
    !
    call gonogo(inRMF_F,go) ! pnS001.rmf or user-selected
    if (go) then
       inSPEC_F = trim(adjustl(prefix)) // "-fovt.pi"
       command = "rmfgen format=var rmfset=" // trim(adjustl(inRMF_F)) // &
            " spectrumset=" // trim(adjustl(inSPEC_F)) // &
            " threshold=1.0e-6 "
       call system(trim(command))
    else
       write(errstr,*) "RMF exists, will not rerun rmfgen here."
       call message(AppMsg,Always)
    end if
    !
    !  Generate the ARF, use a detector map
    !
    call gonogo(inARF_F,go) ! pnS001.arf or user-selected
    ! doesn't matter if fovdef is null string with concatenate
    if (go) then
       command = "evselect table=" // trim(adjustl(inEVLI_F)) // &
            ":EVENTS withfilteredset=yes " // & 
            "expression='(PATTERN<=" // trim(adjustl(pattern)) // &
            "&&(FLAG == 0)" // &
            trim(adjustl(ccddef)) // trim(adjustl(fulldef)) // &
            trim(adjustl(fovdef)) // trim(adjustl(maskitdet)) // &
            "' imagebinning='imageSize' imagedatatype='Int32' " // &
            "imageset=detmap.ds squarepixels=yes ignorelegallimits=no " // &
            "withxranges=yes withyranges=yes xcolumn='DETX' ximagesize=120 " // &
            "ximagemax=19500 ximagemin=-19499 ycolumn='DETY' yimagesize=120 " // &
            "yimagemax=19500 yimagemin=-19499 updateexposure=yes " // &
            "filterexposure=yes"
       call system(trim(command))
       call run_arfgen(inEVLI_F,maskitdet,inSPEC_F,inRMF_F,inARF_F,&
            fovdef,fulldef,ccddef,pattern)
    else
       write(errstr,*) "ARF exists, will not rerun rmfgen here."
       call message(AppMsg,Always)
    end if
    ! repeat in sky coords
    if (dobands) then
       ! Make an image for the region and band in sky coords
       ! OLD: pnS001-clean.fits + pnS001-bkg_region-det.fits = pnS001-obj-im-350-1100.fits
       ! NEW: pnS001-fovevc.fits + pnS001-bkreghsky.fits = pnS001-fovim-350-1100.fits
       inBAND_F = trim(adjustl(prefix)) // "-fovimsky-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       call makeBandImageSky(inEVLI_F,inBAND_F,maskitsky,fulldef,ccddef,quaddefa,fovdef,&
            ruffovdef,celow,cehigh,pattern)
       call gonogo(inBAND_F,go)
       inBAND_F = trim(adjustl(prefix)) // "-fovimskyoot-" // trim(adjustl(celow)) // &
           "-" //  trim(adjustl(cehigh)) // ".fits" ! OOT
       call makeBandImageSky(inEVLIOOT_F,inBAND_F,maskitsky,fulldef,ccddef,quaddefa,fovdef,&
            ruffovdef,celow,cehigh,pattern)
       !
       !  Make an exposure map for the region and band
       inBAND_F = trim(adjustl(prefix)) // "-expimsky-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       !       call gonogo(inBAND_F,go) ???!!! gonogo???
       !       if (go) then
       command = "eexpmap attitudeset=atthk.fits eventset=" // &
            trim(adjustl(inEVLI_F)) // ":EVENTS " // &
            "expimageset=" // trim(adjustl(inBAND_F)) // &
            " imageset=" // trim(adjustl(inIMG_F)) // &
            " pimin=" // trim(adjustl(celow)) // " pimax=" // &
            trim(adjustl(cehigh)) // " withdetcoords=no"
       call system(trim(command))
       ! Make a mask for the region and the band
       ! OLD: pnS001-obj-im.fits -> pnS001-mask-im.fits
       ! NEW: pnS001-fovim(c?).fits -> pnS001-fovmask.fits
       inMASK_F = trim(adjustl(prefix)) // "-maskimsky-" // trim(adjustl(celow)) // &
            "-" // trim(adjustl(cehigh)) // ".fits"
       !       call gonogo(inMASK_F,go) ! mask file pnS001-fovmaskt.fits 
       !if (go) then ???!!! gonogo???
       call run_emask(inMASK_F,inBAND_F)
       !end if
       !
    end if

    do i=1,4
       if (quads(i)) then
          write(schip,'(I1)')i
          write(errstr,*) "Processing by quad, current quad = ",schip
          call message(AppMsg,Always)
          ! Make a spectrum of the observation data for the selected region
          ! OLD: pnS001-1obj.pi NEW: pnS001-fovq1.pi 
          ! ???!!! gonogo???
          !  Extract region spectra separately for each quad
          quadSPEC_F = trim(adjustl(prefix)) // "-fovq" // trim(adjustl(schip)) // &
               ".pi"
          command = "evselect table=" // trim(adjustl(inEVLI_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PATTERN<=" // trim(adjustl(pattern)) // &
               "&&(FLAG == 0)" // trim(adjustl(fulldef)) // trim(adjustl(quaddef(i))) // & 
               trim(adjustl(fovdef)) // trim(adjustl(maskitdet)) // &
               trim(adjustl(ruffovdef)) // "' filtertype=expression keepfilteroutput=no " // &
               "keepfilteroutput=no updateexposure=yes filterexposure=yes " // &
               "withspectrumset=yes spectrumset=" // trim(adjustl(quadSPEC_F)) // &
               " energycolumn=PI spectralbinsize=5 withspecranges=yes " // &
               "specchannelmin=0 specchannelmax=20479"
          call system(trim(command))
          ! run backscale even if exists???!!! gonogo???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLI_F)) // &
               " withbadpixcorr=yes" ! ???!!! ignoreoutoffov=no"
          call system(trim(command))
          !
          !  repeat for OOT
          ! OLD: pnS001-1obj-oot.pi NEW: pnS001-fovootq1.pi
          quadSPEC_F = trim(adjustl(prefix)) // "-fovootq" // trim(adjustl(schip)) // &
               ".pi"
          command = "evselect table=" // trim(adjustl(inEVLIOOT_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PATTERN<=" // trim(adjustl(pattern)) // &
               "&&(FLAG == 0)" // trim(adjustl(fulldef)) // trim(adjustl(quaddef(i))) // & 
               trim(adjustl(fovdef)) // trim(adjustl(maskitdet)) // &
               trim(adjustl(ruffovdef)) // "' filtertype=expression keepfilteroutput=no " // &
               "keepfilteroutput=no updateexposure=yes filterexposure=yes " // &
               "withspectrumset=yes spectrumset=" // trim(adjustl(quadSPEC_F)) // &
               " energycolumn=PI spectralbinsize=5 withspecranges=yes " // &
               "specchannelmin=0 specchannelmax=20479"
          call system(trim(command))
          ! run backscale even if exists???!!! gonogo???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLIOOT_F)) // &
               " withbadpixcorr=yes" ! ???!!! ignoreoutoffov=no"
          call system(trim(command))
          !
          ! Extract a spectrum from the corner pixel data from the observation, 
          ! then backscale it
          ! inCORN_F: OLD: pnS001-corn.fits, NEW pnS001-corevc.fits or corev???
          ! OLD: pnS001-2oc.pi NEW: pnS001-corq2.pi
          quadSPEC_F = trim(adjustl(prefix)) // "-corq" // schip // ".pi"
          call gonogo(quadSPEC_F,go) 
          if (go) then !???!!! no FLAG==0???
             command = "evselect table=" // trim(adjustl(inCORN_F)) // ":EVENTS " // &
                  "withfilteredset=yes expression='(PATTERN<=" // trim(adjustl(pattern)) // &
                  trim(adjustl(quaddef(i))) // trim(adjustl(fulldef)) // &
                  trim(adjustl(corndef)) // "' filtertype=expression keepfilteroutput=no " // &
                  "keepfilteroutput=no withspectrumset=yes spectrumset=" // &
                  trim(adjustl(quadSPEC_F)) // " energycolumn=PI spectralbinsize=5 " // &
                  "withspecranges=yes specchannelmin=0 specchannelmax=20479"
             call system(trim(command))
             ! run backscale even if exists???!!! gonogo???
             command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
                  " badpixlocation="// trim(adjustl(inEVLI_F)) // &
                  " withbadpixcorr=yes ignoreoutoffov=no" !!!??? yet ignore is here???
             call system(trim(command))
          end if
          ! repeat for OOT
          ! OLD: pnS001-1oc-oot.pi NEW pnS0001-corqoot1.pi
          quadSPEC_F = trim(adjustl(prefix)) // "-corootq"  // schip // ".pi"
          call gonogo(quadSPEC_F,go) 
          if (go) then !???!!! no FLAG==0???
             command = "evselect table=" // trim(adjustl(inCORNOOT_F)) // ":EVENTS " // &
                  "withfilteredset=yes expression='(PATTERN<=" // trim(adjustl(pattern)) // &
                  trim(adjustl(quaddef(i))) // trim(adjustl(fulldef)) // &
                  trim(adjustl(corndef)) // "' filtertype=expression keepfilteroutput=no " // &
                  "keepfilteroutput=no withspectrumset=yes spectrumset=" // &
                  trim(adjustl(quadSPEC_F)) // " energycolumn=PI spectralbinsize=5 " // &
                  "withspecranges=yes specchannelmin=0 specchannelmax=20479"
             call system(trim(command))
          end if
          ! run backscale even if exists???!!! gonogo???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLI_F)) // &
               " withbadpixcorr=yes ignoreoutoffov=no" !!!??? yet ignore is here???
          call system(trim(command))
          !
          ! Set up for the FWC data with the selection of appropriate files
          ! Determine the rate [do this with dofilter???]
          !
          command = "evselect table=" // trim(adjustl(inCORN_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='((PI in [600:1300])||(PI in [1650:7200]))" //&
               trim(adjustl(quaddef(i))) // trim(adjustl(fulldef)) // &
               "' filtertype=expression filteredset=temp_events.fits " // &
               " keepfilteroutput=yes updateexposure=yes filterexposure=yes "
          call system(trim(command))
          command = "ls -l temp_events.fits"
          call system(trim(command))
          !
          ! OLD fkeypar section (call read_header_info in gofio instead)
          ! get these from new temp_events.fits NAXIS1, expo, LIVETInn
          !
          status = 0
          warndie = .TRUE.
          call check_open_set("temp_events.fits",temp_S,1,status,warndie)
          call getkeys(temp_S,cnts,expo,livetime,schip)
          call release(temp_S)
          !
          ! Repeat for OOT, but just need cntsoot [do this with dofilter???]
          call unlink ("temp_events.fits")
          command = "evselect table=" // trim(adjustl(inCORNOOT_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='((PI in [600:1300])||(PI in [1650:7200]))" //&
               trim(adjustl(quaddef(i))) //  trim(adjustl(fulldef)) // &
               "' filtertype=expression filteredset=temp_events.fits " // &
               " keepfilteroutput=yes updateexposure=yes filterexposure=yes "
          call system(trim(command))
          command = "ls -l temp_events.fits"
          call system(trim(command))
          !
          ! OLD fkeypar section (call read_header_info in gofio instead)
          ! get these from new temp_events.fits NAXIS1, expo, LIVETInn
          !
          status = 0
          warndie = .TRUE.
          call check_open_set("temp_events.fits",temp_S,1,status,warndie)
          command = "ls -l temp_events.fits"
          call system(trim(command))
          call getkeys(temp_S,cntsoot,expooot,livetime,schip)
          rate = (cnts-(scale * cntsoot))/((1.0-scale) * livetime)
          rate = rate * 100.
          call release(temp_S)
          command = "ls -l temp_events.fits"
          call system(trim(command))
          !
          ! Determine the hardness ratio
          !
          call unlink ("temp_events.fits")
          command = "evselect table=" // trim(adjustl(inCORN_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PI in [600:1300])' " // &
               "filtertype=expression filteredset=temp_events.fits " // &
               "keepfilteroutput=yes updateexposure=yes filterexposure=yes "
          call system(trim(command))
          ! return lowe (NAXIS2), expo, LIVETInn
          call check_open_set("temp_events.fits",temp_S,1,status,warndie)
          call getkeys(temp_S,ilo,expo,livetime,schip)
          call release(temp_S)
          call unlink ("temp_events.fits")   
          command = "evselect table=" // trim(adjustl(inCORN_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PI in [1650:7200])' " // &
               "filtertype=expression filteredset=temp_events.fits " // &
               "keepfilteroutput=yes updateexposure=yes filterexposure=yes "
          call system(trim(command))
          ! return hige (NAXIS2), expo, LIVETInn
          call check_open_set("temp_events.fits",temp_S,1,status,warndie)
          call getkeys(temp_S,ihi,expo,livetime,schip)
          call release(temp_S)
          call unlink ("temp_events.fits")
          hige = real(ihi)
          lowe = real(ilo)
          ! repeat for OOT
          command = "evselect table=" // trim(adjustl(inCORNOOT_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PI in [600:1300])' " // &
               "filtertype=expression filteredset=temp_events.fits " // &
               "keepfilteroutput=yes updateexposure=yes filterexposure=yes "

          call system(trim(command))
          ! return lowe (NAXIS2), expo, LIVETInn
          call check_open_set("temp_events.fits",temp_S,1,status,warndie)
          call getkeys(temp_S,ilo,expo,livetime,schip)
          call release(temp_S)
          call unlink ("temp_events.fits")   
          command = "evselect table=" // trim(adjustl(inCORNOOT_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PI in [1650:7200])' " // &
               "filtertype=expression filteredset=temp_events.fits " // &
               "keepfilteroutput=yes updateexposure=yes filterexposure=yes "
          call system(trim(command))
          ! return hige (NAXIS2), expo, LIVETInn
          call check_open_set("temp_events.fits",temp_S,1,status,warndie)
          call getkeys(temp_S,ihi,expo,livetime,schip)
          call release(temp_S)
          call unlink ("temp_events.fits")
          higeoot = real(ihi)
          loweoot = real(ilo)
          !
          hard = (hige-(scale*higeoot))/(lowe-(scale*loweoot))
          ! no hardsig???!!!
          !hardsig=sqrt((hige/(lowe*lowe))+((hige*hige*lowe)/(lowe*lowe*lowe*lowe)))
          write(errstr,*)  " hige, scale, higeoot, scale, loweoot, = ",&
               hige, scale, higeoot, scale, loweoot
          call message(AppMsg,Always)
          write(errstr,*)  " Corner rate     = ",rate
          call message(AppMsg,Always)
          write(errstr,*)  " Corner hardness = ",hard
          call message(AppMsg,Always)

          ! Get the FWC spectrum from the region and backscale it
          ! OLD: pnS001-2ff.pi NEW: pnS001-fovfwcq2.pi
          quadSPEC_F = trim(adjustl(prefix)) // "-fovfwcq"  // schip // ".pi"
          !          call gonogo(ccdSPEC_F,go) ???!!! no gonogo???
          !          if (go) then
          command = "evselect table=" // trim(adjustl(inFWC_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PATTERN<=" // &
               trim(adjustl(pattern)) // "&&(FLAG == 0)" // &
               trim(adjustl(ruffovdef)) // trim(adjustl(quaddef(i))) // &
               trim(adjustl(fovdef)) //  trim(adjustl(fulldef)) // &
               trim(adjustl(maskitdet)) // "' energycolumn=PI spectralbinsize=5 " // &
               " withspectrumset=yes spectrumset=" // trim(adjustl(quadSPEC_F)) // &
               " specchannelmin=0 specchannelmax=20479"
          call system(trim(command))
          !          end if
          ! run backscale even if exists???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLI_F)) // &
               " withbadpixcorr=yes"
          call system(trim(command))
          !
          ! repeat-ish for OOT
          ! OLD: pnS001-2ff-oot.pi NEW: pnS001-fovfwcootq2.pi
          ! hardcode to testprods???!!!
          quadSPEC_F = trim(adjustl(prefix)) // "-fovfwcootq"  // schip // ".pi"
          !          call gonogo(ccdSPEC_F,go) ???!!! no gonogo???
          !          if (go) then
          command = "evselect table=" // trim(adjustl(inFWC_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PATTERN<=" // &
               trim(adjustl(pattern)) // "&&(FLAG == 0)" // & !"&&(OOT == 1)"
               trim(adjustl(ruffovdef)) // trim(adjustl(quaddef(i))) // &
               trim(adjustl(fovdef)) //  trim(adjustl(fulldef)) // &
               trim(adjustl(maskitdet)) // "' energycolumn=PI spectralbinsize=5 " // &
               " withspectrumset=yes spectrumset=" // trim(adjustl(quadSPEC_F)) // &
               " specchannelmin=0 specchannelmax=20479" 
          call system(trim(command))
!          end if
          ! run backscale even if exists???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLIOOT_F)) // &
               " withbadpixcorr=yes"
          call system(trim(command))
          !  Extract a spectrum from the corner pixel data from the FWC data, 
          !  then backscale it
          ! OLD: pnS001-2fc.pi NEW: pnS001-corfwcq2.pi
          ! hardcode to testprods???!!!
          quadSPEC_F = trim(adjustl(prefix)) // "-corfwcq"  // schip // ".pi"
          !          call gonogo(ccdSPEC_F,go) ???!!! no gonogo???
          !          if (go) then
          command = "evselect table=" // trim(adjustl(inFWC_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PATTERN<=" // &
               trim(adjustl(pattern)) // "&&((FLAG & 0x762a097c)==0)" // &
               trim(adjustl(corndef)) // trim(adjustl(quaddef(i))) // &
               trim(adjustl(fulldef)) // "' energycolumn=PI spectralbinsize=5 " // &
               "withspectrumset=yes spectrumset=" // trim(adjustl(quadSPEC_F)) // &
               " specchannelmin=0 specchannelmax=20479"
          call system(trim(command))
          !          end if
          ! run backscale even if exists???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLI_F)) // &
               " withbadpixcorr=yes ignoreoutoffov=no "
          call system(trim(command))
          !
          ! repeat for OOT
          ! OLD: pnS001-2fc-oot.pi NEW: pnS001-corfwcootq2.pi
          ! hardcode to testprods???!!!
          quadSPEC_F = trim(adjustl(prefix)) // "-corfwcootq"  // schip // ".pi"
          !          call gonogo(ccdSPEC_F,go) ???!!! no gonogo???
          !          if (go) then
          command = "evselect table=" // trim(adjustl(inFWC_F)) // ":EVENTS " // &
               "withfilteredset=yes expression='(PATTERN<=" // & !"&&(OOT == 1)"
               trim(adjustl(pattern)) // "&&((FLAG & 0x762a097c)==0)" // &
               trim(adjustl(corndef)) // trim(adjustl(quaddef(i))) // &
               trim(adjustl(fulldef)) // "' energycolumn=PI spectralbinsize=5 " // &
               "withspectrumset=yes spectrumset=" // trim(adjustl(quadSPEC_F)) // &
               " specchannelmin=0 specchannelmax=20479"
          call system(trim(command))
          !          end if
          ! run backscale even if exists???
          command = "backscale spectrumset=" //  trim(adjustl(quadSPEC_F)) // &
               " badpixlocation="// trim(adjustl(inEVLI_F)) // &
               " withbadpixcorr=yes ignoreoutoffov=no "
          call system(trim(command))
          !
          if (dobands) then
             ! Make an image of the fwc data in the requested band, if a band is selected
             ! gonogo???
             ! OLD: pnS001-im2-350-1100.fits NEW: pnS001-fwcimq2det-350-1100.fits
             quadIMG_F = trim(adjustl(prefix)) // "-fwcimq"  // schip // &
                  "det" // "-" // trim(adjustl(celow)) // "-" // &
                  trim(adjustl(cehigh)) // ".fits"
             
             command = "evselect table=" // trim(adjustl(inFWC_F)) // ":EVENTS " // &
                  "withfilteredset=yes expression='(PATTERN <= " // &
                  trim(adjustl(pattern)) // "&&(FLAG == 0)&&(PI in [" // &
                  trim(adjustl(celow)) // ":" // trim(adjustl(cehigh)) // "])" // &
                  trim(adjustl(quaddef(i))) // trim(adjustl(fovdef)) // &
                  trim(adjustl(fulldef)) // trim(adjustl(maskitdet)) // &
                  "' imagebinning='imageSize' imagedatatype='Int32' " // &
                  "imageset=" // trim(adjustl(quadIMG_F))  // " squarepixels=yes " // &
                  "withxranges=yes withyranges=yes xcolumn='DETX' ximagesize=780 " // &
                  "ximagemax=19500 ximagemin=-19499 ycolumn='DETY' yimagesize=780 " // &
                  "yimagemax=19500 yimagemin=-19499 updateexposure=yes filterexposure=yes " // &
                  "ignorelegallimits=yes "
             call system(trim(command))
             ! Mask the FWC image [check this???!!!]
             ! OLD: pnS001-mask-im-det-350-1100.fits NEW: pnS001-maskimqdet
             maskIMG_F = trim(adjustl(prefix)) // "-maskimq"  // schip // &
                  "det" // "-" // trim(adjustl(celow)) // "-" // &
                  trim(adjustl(cehigh)) // ".fits"

             command = "farith " // trim(adjustl(quadIMG_F)) // "'" // &
                  trim(adjustl(maskIMG_F)) // "[MASK]' " // &
                  trim(adjustl(inMASK_F)) // " MUL copyprime=yes"
             !
             ! Rename the FWC image (after masking)
             call rename (trim(adjustl(maskIMG_F)),trim(adjustl(quadIMG_F)))
             !
             ! Make an OOT image of the fwc data in the requested band
             ! gonogo???!!!
             ! OLD: pnS001-im2-350-1100.fits NEW: pnS001-fwcimq2det-350-1100.fits
             quadIMG_F = trim(adjustl(prefix)) // "-fwcimootq"  // schip // &
                  "det" // "-" // trim(adjustl(celow)) // "-" // &
                  trim(adjustl(cehigh)) // ".fits"
             command = "evselect table=" // trim(adjustl(inFWC_F)) // ":EVENTS " // &
                  "withfilteredset=yes expression='(PATTERN <= " // &
                  trim(adjustl(pattern)) // "&&(FLAG == 0)&&(PI in [" // &
                  trim(adjustl(celow)) // ":" // trim(adjustl(cehigh)) // "])" // &
                  trim(adjustl(quaddef(i))) // trim(adjustl(fovdef)) // &
                  trim(adjustl(fulldef)) // trim(adjustl(maskitdet)) // &
                  "' imagebinning='imageSize' imagedatatype='Int32' " // &
                  "imageset=" // trim(adjustl(quadIMG_F))  // " squarepixels=yes " // &
                  "withxranges=yes withyranges=yes xcolumn='DETX' ximagesize=780 " // &
                  "ximagemax=19500 ximagemin=-19499 ycolumn='DETY' yimagesize=780 " // &
                  "yimagemax=19500 yimagemin=-19499 updateexposure=yes " // &
                  "filterexposure=yes ignorelegallimits=yes "
             call system(trim(command))
             ! Mask the FWC image [check this???!!!]
             ! OLD: pnS001-mask-im-det-350-1100.fits NEW: pnS001-maskimqNdet
             maskIMG_F = trim(adjustl(prefix)) // "-maskimootq"  // schip // &
                  "det" // "-" // trim(adjustl(celow)) // "-" // &
                  trim(adjustl(cehigh)) // ".fits"
             ! replace with CAL/DAL ???!!!
             command = "farith " // trim(adjustl(quadIMG_F)) // "'" // &
                  trim(adjustl(maskIMG_F)) // "[MASK]' " // &
                  trim(adjustl(inMASK_F)) // " MUL copyprime=yes"
             ! Rename the FWC image (after masking)
             call rename (trim(adjustl(maskIMG_F)),trim(adjustl(quadIMG_F)))
          endif
       end if
    end do
    !
    ! clean up (add things here???!!!)
    if (.not.keep) then
       write(errstr,*) "PNSPECTRA: keepinterfiles NOT set"
       call message(AppMsg,Always)
       write(errstr,*) "PNSPECTRA: Deleting intermediary files..."
       call message(AppMsg,Always)
       write(errstr,*) "Deleting detmap.ds"
       call message(AppMsg,Always)
       call unlink("detmap.ds")
       write(errstr,*) "Deleting filtered.fits"
       call message(AppMsg,Always)
       call unlink("filtered.fits")
!       write(errstr,*) "Deleting temp_events.fits"
!       call message(AppMsg,Always)
!       call unlink("temp_events.fits")
    end if

    return

  end subroutine pnspectra
  !
  !
  !
  subroutine run_emask(detmask,expimg)
    !
    ! Makes masks files
    !
    character(len=*), intent(in) :: detmask, expimg
    character(len=1024) :: command
    command = "emask detmaskset=" // trim(adjustl(detmask)) // & 
         " expimageset=" // trim(adjustl(expimg)) // & 
         " threshold1=0.01 threshold2=0.5 verbosity=4"
    write(errstr,*) " Running emask command = "
    call message(AppMsg,Verbose)
    write(errstr,*) trim(adjustl(command))
    call message(AppMsg,Verbose)
    call system(trim(command))

    return

  end subroutine run_emask

  subroutine run_arfgen(evli,maskit,spec,rmf,arf,fovdef,fulldef,quaddef,pattern)
    !
    character(len=*) :: evli,maskit,spec,rmf,arf,fovdef,fulldef,quaddef,pattern
    character(len=1024) :: command

    write(errstr,*) "RUN_ARFGEN: Running evselect, then arfgen..."
    call message(AppMsg,Always)

    command = "evselect table=" // trim(adjustl(evli)) // &
         ":EVENTS withfilteredset=yes " // &
         "expression='(PATTERN <= " // trim(adjustl(pattern)) // &
         "&&(FLAG == 0)" // &
         trim(adjustl(fovdef)) // trim(adjustl(fulldef)) // &
         trim(adjustl(quaddef)) // trim(adjustl(maskit)) // &
         "' imagebinning='imageSize' imagedatatype='Int32' " // &
         "imageset=detmap.ds squarepixels=yes ignorelegallimits=no " // &
         "withxranges=yes withyranges=yes xcolumn='DETX' ximagesize=120 " // &
         "ximagemax=19500 ximagemin=-19499 ycolumn='DETY' yimagesize=120 " // &
         "yimagemax=19500 yimagemin=-19499 updateexposure=yes filterexposure=yes "
    call system(trim(command))
    command = "arfgen arfset=" // trim(adjustl(arf)) // " spectrumset=" // &
         trim(adjustl(spec)) //  " withrmfset=yes " // &
         " rmfset=" // trim(adjustl(rmf)) // " extendedsource=yes " // &
         "modelee=no withbadpixcorr=no detmaptype=dataset detmaparray=detmap.ds " // &
         " badpixlocation=" // trim(adjustl(evli)) // " modelootcorr=no "
    write(errstr,*) "RUN_ARFGEN: arfgen command = " // trim(adjustl(command))
    call message(AppMsg,Verbose)
    call system(trim(command))

    return

  end subroutine run_arfgen
  !
  !
  !
  subroutine printParams(inEVLI_F,inOOT_F,inCORN_F,inCORNOOT_F,inIMG_F,inIMGOOT_F,&
       inSPMASK_F,inEXPMAP_F,inMASK_F,withsky,inSKY_F,withdet,inDET_F,withreg,&
       inREG_F,inRMF_F,inARF_F,quads,elow,ehigh,dobands,keep)

    character(len=*) :: inEVLI_F,inOOT_F,inCORN_F,inCORNOOT_F,inIMG_F,inIMGOOT_F,&
         inSPMASK_F,inEXPMAP_F,inMASK_F,inSKY_F,inDET_F,inREG_F,inRMF_F,inARF_F
    integer :: elow, ehigh, i
    logical :: withsky, withdet, withreg, dobands, keep, quads(4)

    write(errstr,*) "Input Cleaned FOV event list:         ",trim(adjustl(inEVLI_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Cleaned FOV OOT event list:     ",trim(adjustl(inOOT_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Cleaned CORNER event list:      ",trim(adjustl(inCORN_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Cleaned CORNER OOT event list:  ",trim(adjustl(inCORNOOT_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Cleaned IMAGE:                  ",trim(adjustl(inIMG_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Cleaned OOT IMAGE:              ",trim(adjustl(inIMGOOT_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input SP scaling mask:                ",trim(adjustl(inSPMASK_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Exposure Map:                   ",trim(adjustl(inEXPMAP_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Band and Region mask :          ",trim(adjustl(inMASK_F))
    call message(AppMsg,Always)
    if (withsky) then
       write(errstr,*) "Input SKY coord region mask:       ",trim(adjustl(inSKY_F))
       call message(AppMsg,Always)
    endif
    if (withdet) then
       write(errstr,*) "Input DET coord region mask:       ",trim(adjustl(inDET_F))
       call message(AppMsg,Always)
    endif
    if (withreg) then
       write(errstr,*) "Input text region file:            ",trim(adjustl(inREG_F))
       call message(AppMsg,Always)
    endif
    write(errstr,*) "Input Response Matrix (RMF) file:     ",trim(adjustl(inRMF_F))
    call message(AppMsg,Always)
    write(errstr,*) "Input Aux Response Matrix (ARF) file: ",trim(adjustl(inARF_F))
    call message(AppMsg,Always)
    write(errstr,*) "QUADs selected: ",quads
    call message(AppMsg,Always)
    write(errstr,*) "Low energy threshold (eV):  ",elow
    call message(AppMsg,Always)
    write(errstr,*) "High energy threshold (eV): ",ehigh
    call message(AppMsg,Always)
    write(errstr,*) " Do bands (apply ehigh, elow?): ",dobands
    call message(AppMsg,Always)
    write(errstr,*) "Keep intermediary files?: ",keep
    call message(AppMsg,Always)
    
    return

  end subroutine printparams
  !
  ! Define all the selection regions for evselect, etc
  !
  subroutine defineQUADreg (quads,quaddef)
    
    !  Set up the quad selection expression, data are processed in quadrants
    
    character(len=*), intent(inout) :: quaddef
    logical, intent(in) :: quads(4)
    quaddef = '&&('

    if(quads(1)) then
       quaddef = trim(adjustl(quaddef)) // "(CCDNR == 1)||(CCDNR == 2)||(CCDNR == 3)"
       if(quads(2)) then
          quaddef = trim(adjustl(quaddef)) // "||(CCDNR == 4)||(CCDNR == 5)||(CCDNR == 6)"
       endif
       if(quads(3)) then
          quaddef = trim(adjustl(quaddef)) // "||(CCDNR == 7)||(CCDNR == 8)||(CCDNR == 9)"
       endif
       if(quads(4)) then
          quaddef = trim(adjustl(quaddef)) // "||(CCDNR == 10)||(CCDNR == 11)||(CCDNR == 12)"
       endif
    elseif(quads(2)) then
       quaddef = trim(adjustl(quaddef)) // "(CCDNR == 4)||(CCDNR == 5)||(CCDNR == 6)"
       if(quads(3)) then
          quaddef = trim(adjustl(quaddef)) // "||(CCDNR == 7)||(CCDNR == 8)||(CCDNR == 9)"
       endif
       if(quads(4)) then
          quaddef = trim(adjustl(quaddef)) // "||(CCDNR == 10)||(CCDNR == 11)||(CCDNR == 12)"
       endif
    elseif(quads(3)) then
       quaddef = trim(adjustl(quaddef)) // "(CCDNR == 7)||(CCDNR == 8)||(CCDNR == 9)"
       if(quads(4)) then
          quaddef = trim(adjustl(quaddef)) // "||(CCDNR == 10)||(CCDNR == 11)||(CCDNR == 12)"
       end if
    elseif(quads(4)) then
       quaddef = trim(adjustl(quaddef)) // "(CCDNR == 10)||(CCDNR == 11)||(CCDNR == 12)"
    endif

    quaddef = trim(adjustl(quaddef)) // ")"
    write (errstr,*) "Quad selection expression (quaddef): ",trim(adjustl(quaddef))
    call message(AppMsg,Noisy) ! change to Noisy???!!!

    return

  end subroutine defineQUADreg
  !
  !
  !
  subroutine defineexpr (cornerdef,quaddef,fulldef,ruffovdef)
    ! Definitions
    !   quaddef is the selection criteria for the active CCDs, i.e., 
    !        those not excluded by the quads parameter
    !   corndef is a definition of the corner pixels - it excludes
    !        the open FOV of the detector
    !   quaddef is a definition of the location of each quadrant
    !        in DETX,DETY space
    !   fulldef is a definition of box containing the full CCD area
    !        and provides no additional restrictions, it just allows
    !        backscale to be happy when calculating the BACKSCAL
    !        keyword for the corner spectra.
    !   ruffovdef is the circle restricting the FOV to the area
    !        open to the sky.
    !   psdef is the point-source exclusion as selected by the input
    !        parameter. check this ???!!!
    !
    character(len=*), intent(inout) :: cornerdef, fulldef, ruffovdef, quaddef(4)
    cornerdef = ''
    fulldef = ''
    ruffovdef = ''
    quaddef = ''
    !    
    write (errstr,*) "Defining corner selection expression for EPN"
    call message(AppMsg,Verbose)
    cornerdef = "&&!((DETX,DETY) IN circle(-2200,-1110,18200))"
    quaddef = (/"&&((DETX,DETY) in BOX(-10241.5,7115.0,8041.5,8210.0,0))   ",&
                "&&((DETX,DETY) in BOX(5840.0,7115.0,8040.0,8210.0,0))     ",&
                "&&((DETX,DETY) in BOX(5840.0,-9311.0,8040.0,8216.0,0))    ",&
                "&&((DETX,DETY) in BOX(-10241.5,-9311.0,8041.5,8216.0,0))  " /)
    fulldef =   "&&((DETX,DETY) in BOX(-2196,-1110,16060,15510,0))"
    ruffovdef = "&&((DETX,DETY) IN circle(-2200,-1110,17980))"
    
    return

  end subroutine defineexpr

  subroutine read_region(regfile,region,withregion)
    ! read the text region file to create fovdef
    ! Get the definition of the object region
    character(len=*) :: regfile, region
    logical :: i_ok = .FALSE.
    logical, intent(inout) :: withregion
    integer :: ios, lun
    region = ""
    ios = 0
    inquire(FILE=trim(regfile), EXIST=i_ok)
    if (i_ok) then
       write(errstr,*) "Region file exists: ", &
            trim(regfile)
       call message(AppMsg,Noisy)
       lun = GetLun() ! in gofutils
       open (lun,file=trim(regfile), iostat=ios)
       if (ios .ne. 0) then
          call warning ("noLUN","Cannot open region file!")
          region = ""
          withregion = .FALSE.
       end if
       ! sanity check here ???!!!
       read(lun,'(A)')region
       write(errstr,*) "Area selection from region text file: ",&
            trim(adjustl(region))
       call message(AppMsg,Noisy)
    else
       write(errstr,*)&
            "Region file selected does not exist, defaulting to no additional selection."
       call message(AppMsg,Always)
       region = ""
       withregion = .FALSE.
    end if
    return

  end subroutine read_region
  !
  !
  !
  subroutine docorner(inE_F,inO_F,inC_F,inOC_F,pattern,quaddef,corndef)
    
    character(len=*) :: inE_F, inC_F, inO_F, inOC_F, pattern, quaddef, corndef
    character(len=1024) :: command
    logical :: i_ok
    integer :: ios, lun
    ios = 0
    i_ok = .FALSE. 
    inquire(FILE=trim(inC_F), EXIST=i_ok)
    if (i_ok) then
       write(errstr,*) "Corner event list exists, evselect NOT rerun."
       call message(AppMsg,Always)
    else
       write(errstr,*) " CORNER eventlist evselect command = "
       call message(AppMsg,Verbose)
       command = "evselect table=" // trim(adjustl(inE_F)) // ":EVENTS " //&
            "withfilteredset=yes expression='(PATTERN<=" // trim(adjustl(pattern)) // &
            trim(adjustl(quaddef)) // "&&((FLAG & 0x762a097c) == 0)" // &
            trim(adjustl(corndef)) // "' filtertype=expression keepfilteroutput=yes " // &
            "updateexposure=yes filterexposure=yes filteredset=" // trim(adjustl(inC_F))
       write(errstr,*) "Corner-only evselect command = "
       call message(AppMsg,Verbose)
       write(errstr,*) trim(adjustl(command))
       call message(AppMsg,Verbose)
       call system(trim(command))
    end if
    ! Now do the OOT corner data
    ios = 0
    i_ok = .FALSE. 
    inquire(FILE=trim(inOC_F), EXIST=i_ok)
    if (i_ok) then
       write(errstr,*) "Corner OOT event list exists, evselect NOT rerun."
       call message(AppMsg,Always)
    else
       write(errstr,*) " CORNER OOT eventlist evselect command = "
       call message(AppMsg,Verbose)
       command = "evselect table=" // trim(adjustl(inO_F)) // ":EVENTS " //&
            "withfilteredset=yes expression='(PATTERN<=" // trim(adjustl(pattern)) // &
            trim(adjustl(quaddef)) // "&&((FLAG & 0x762a097c) == 0)" // &
            trim(adjustl(corndef)) // "' filtertype=expression keepfilteroutput=yes " // &
            "updateexposure=yes filterexposure=yes filteredset=" // trim(adjustl(inOC_F))
       write(errstr,*) "Corner-only OOT evselect command = "
       call message(AppMsg,Verbose)
       write(errstr,*) trim(adjustl(command))
       call message(AppMsg,Verbose)
       call system(trim(command))
    end if
    
    return
    
  end subroutine docorner
  !
  !
  !
  subroutine makeImFile(evlist,imfile,maskit,pattern,quaddefa,fulldef,ruffovdef)
    !
    ! Make the clean FOV image files in sky coords
    ! OLD: pnS001-obj-im.fits
    ! NEW: pnS001-fovimt.fits (same as cheese call???!!!)
    !
    character(len=*), intent(in) :: evlist, imfile, maskit, quaddefa, &
         fulldef, ruffovdef, pattern
    character(len=1024) :: command
    integer :: elow, ehigh
    logical :: i_ok
    command = ""

    write(errstr,*) "Making FOV image file."
    call message(AppMsg,Verbose)
    inquire(FILE=trim(imfile), EXIST=i_ok)
    if (i_ok) then
       if ( clobber() ) then
          write(errstr,*) " FOV image file: ",trim(adjustl(imfile)),&
               " exists and CLOBBER is set, will overwrite existing file."
          call message(AppMsg,Always)          
          call unlink (trim(adjustl(imfile)))
       else
          write(errstr,*) " FOV image file: ",trim(adjustl(imfile)),&
               " exists and CLOBBER is NOT set, using existing..."
          call message(AppMsg,Always)
          return
       endif
    end if ! check this ???!!! Real32 vs Int32???
    command = "evselect table=" // trim(adjustl(evlist)) // &
         ":EVENTS withfilteredset=yes expression=" // & 
         "'(PATTERN<=" // trim(adjustl(pattern)) // "&&(FLAG == 0)" // &
         trim(adjustl(quaddefa)) // trim(adjustl(fulldef)) // & 
         trim(adjustl(ruffovdef)) // trim(adjustl(maskit)) // &
         "&&(PI in [400:7200])' filtertype=expression " // &
         "imagebinning='imageSize' imagedatatype='Int32' imageset=" // &
         trim(adjustl(imfile)) // " squarepixels=yes ignorelegallimits=yes " // &
         "withxranges=yes withyranges=yes xcolumn='X' ximagesize=900 " // &
         "ximagemax=48400 ximagemin=3401 ycolumn='Y' yimagesize=900 " // &
         "yimagemax=48400 yimagemin=3401 updateexposure=yes filterexposure=yes"
    write(errstr,*) " FOV image evselect command = "
    call message(AppMsg,Verbose)
    write(errstr,*) trim(adjustl(command))
    call message(AppMsg,Verbose)
    call system(trim(adjustl(command)))

    return

  end subroutine makeImFile
  !
  !
  !
  subroutine makeSPfile(evlist,infile,maskit,fulldef,quaddef,quaddefa,fovdef,&
       ruffovdef,celow,cehigh,pattern)
    !
    ! Extract the entire FOV region from observation data into an image
    ! Do it again but use all events.  This will be a mask for calculating
    ! the SP scaling
    !
    ! file will be pnS001-fovimspdet.fits
    !
    character(len=*), intent(in) :: evlist, infile, maskit, ruffovdef, &
         quaddef, quaddefa, fulldef, fovdef, celow, cehigh, pattern
    character(len=1024) :: command
    integer :: elow, ehigh
    logical :: i_ok

    write(errstr,*) "Making SP DET image file..."
    call message(AppMsg,Verbose)
    inquire(FILE=trim(infile), EXIST=i_ok)
    if (i_ok) then
       if ( clobber() ) then
          write(errstr,*) " FOV SP DET file: ",trim(infile),&
               " exists and CLOBBER is set, will overwrite existing file."
          call message(AppMsg,Always)          
          call unlink (trim(infile))
       else
          write(errstr,*) " FOV SP DET file: ",trim(infile),&
               " exists and CLOBBER is NOT set, using existing..."
          call message(AppMsg,Always)
          return
       endif
    end if ! check this???!!!
    command = "evselect table=" // trim(adjustl(evlist)) // &
         ":EVENTS withfilteredset=yes expression=" // & 
         "'(PATTERN<=" // trim(adjustl(pattern)) // "&&(FLAG == 0)" // &
         trim(adjustl(fulldef)) // trim(adjustl(quaddef)) // trim(adjustl(FOVdef)) // &
         trim(adjustl(ruffovdef)) // trim(adjustl(maskit)) // "' " // &
         "filtertype=expression " // "imagebinning='imageSize' imagedatatype='Int32' " // &
         "imageset=" // trim(adjustl(infile)) // " squarepixels=yes " // &
         "ignorelegallimits=yes withxranges=yes withyranges=yes xcolumn='DETX' " // &
         "ximagesize=780 ximagemax=19500 ximagemin=-19499 ycolumn='DETY' " // &
         "yimagesize=780 yimagemax=19500 yimagemin=-19499 updateexposure=yes " // &
         "filterexposure=yes"
    write(errstr,*) " FOV SP DET evselect command = "
    call message(AppMsg,Verbose)
    write(errstr,*) trim(adjustl(command))
    call message(AppMsg,Verbose)
    call system(trim(command))

    return

  end subroutine makeSPfile

  subroutine makeBandImageSky (evlist,infile,maskit,fulldef,ccddef,quaddefa,fovdef,&
       ruffovdef,celow,cehigh,pattern)
    !
    ! Extract the entire FOV region and band from observation data into an 
    ! image in detector coords (old: pnS001-obj-im-det-350-1100.fits
    ! file will be pnS001-fovim-350-1100-det.fits
    !
    character(len=*), intent(in) :: evlist, infile, maskit, ruffovdef, pattern
    character(len=*), intent(inout) :: ccddef, quaddefa, fulldef, fovdef, celow, cehigh
    character(len=1024) :: command
    logical :: i_ok

    write(errstr,*) "Making band image file in SKY coords..."
    call message(AppMsg,Verbose)
    inquire(FILE=trim(infile), EXIST=i_ok)
    if (i_ok) then
       if ( clobber() ) then
          write(errstr,*) " Band image: ",trim(adjustl(infile)),&
               " exists and CLOBBER is set, will overwrite existing file."
          call message(AppMsg,Always)          
          call unlink (trim(infile))
       else
          write(errstr,*) "Band image: ",trim(adjustl(infile)),&
               " exists and CLOBBER is NOT set, using existing..."
          call message(AppMsg,Always)
          return
       endif
    end if
    command = "evselect table="// trim(adjustl(evlist)) // &
         ":EVENTS withfilteredset=yes filtertype=expression expression=" // &
         "'(PATTERN<=" // trim(adjustl(pattern)) // "&&(FLAG == 0)" // &
         "&&(PI in [" // trim(adjustl(celow)) // ":" // trim(adjustl(cehigh)) // &
         "])" // trim(adjustl(ccddef)) // trim(adjustl(fulldef)) // &
         trim(adjustl(fovdef)) // trim(adjustl(ruffovdef)) // &
         trim(adjustl(maskit)) // "' imagebinning='imageSize' " // &
         "imagedatatype='Int32' imageset=" // trim(adjustl(infile)) // &
         " squarepixels=yes ignorelegallimits=yes withxranges=yes " // &
         "withyranges=yes xcolumn='X' ximagesize=900 ximagemax=48400 " // &
         "ximagemin=3401 ycolumn='Y' yimagesize=900 yimagemax=48400 " // &
        "yimagemin=3401 updateexposure=yes filterexposure=yes"

    write(errstr,*) " Band image evselect command = "
    call message(AppMsg,Verbose)
    write(errstr,*) trim(adjustl(command))
    call message(AppMsg,Verbose)
    call system(trim(command))

    return

  end subroutine makeBandImageSky
  
  subroutine makeBandImageDet (evlist,infile,maskit,fulldef,quaddef,quaddefa,fovdef,&
       ruffovdef,celow,cehigh,pattern)
    !
    ! Extract the entire FOV region and band from observation data into an 
    ! image in detector coords (old: pnS001-obj-im-det-350-1100.fits
    ! file will be pnS001-fovim-350-1100-det.fits
    !
    character(len=*), intent(in) :: evlist, infile, maskit, ruffovdef, pattern
    character(len=*), intent(inout) :: quaddef, quaddefa, fulldef, fovdef, celow, cehigh
    character(len=1024) :: command
    logical :: i_ok

    write(errstr,*) "Making FOV band image file..."
    call message(AppMsg,Verbose)
    inquire(FILE=trim(infile), EXIST=i_ok)
    if (i_ok) then
       if ( clobber() ) then
          write(errstr,*) " FOV region and band image: ",trim(infile),&
               " exists and CLOBBER is set, will overwrite existing file."
          call message(AppMsg,Always)          
          call unlink (trim(infile))
       else
          write(errstr,*) "FOV region and band image: ",trim(infile),&
               " exists and CLOBBER is NOT set, using existing..."
          call message(AppMsg,Always)
          return
       endif
    end if
    command = "evselect table="// trim(adjustl(evlist)) // &
         ":EVENTS withfilteredset=yes filtertype=expression expression=" // &
         "'(PATTERN<=" // trim(adjustl(pattern)) // "&&(FLAG == 0)" // &
         "&&(PI in [" // trim(adjustl(celow)) // ":" // trim(adjustl(cehigh)) // &
         "])" // trim(adjustl(quaddef)) // trim(adjustl(fulldef)) // &
         trim(adjustl(fovdef)) // trim(adjustl(ruffovdef)) // &
         trim(adjustl(maskit)) // "' imagebinning='imageSize' " // &
         "imagedatatype='Int32' imageset=" // trim(adjustl(infile)) // &
         " squarepixels=yes ignorelegallimits=yes withxranges=yes " // &
         "withyranges=yes xcolumn='DETX' ximagesize=780 ximagemax=19500 " // &
         "ximagemin=-19499 ycolumn='DETY' yimagesize=780 yimagemax=19500 " // &
         "yimagemin=-19499 updateexposure=yes filterexposure=yes"

    write(errstr,*) " FOV region and band image evselect command = "
    call message(AppMsg,Verbose)
    write(errstr,*) trim(adjustl(command))
    call message(AppMsg,Verbose)
    call system(trim(command))

    return

  end subroutine makeBandImageDet

  subroutine gonogo(file,go)

    character(len=*) :: file
    logical, intent(inout) :: go 
    logical :: i_ok
    i_ok = .FALSE.
    go = .TRUE.
    inquire(FILE=trim(adjustl(file)), EXIST=i_ok)
    if (i_ok) then
       if ( clobber() ) then
          write(errstr,*) " File: ",trim(file),&
               " exists and CLOBBER is set, will overwrite existing file."
          call message(AppMsg,Always)          
          call unlink (trim(file))
       else
          write(errstr,*)  " File: ",trim(file),&
               " exists and CLOBBER is NOT set, using existing..."
          call message(AppMsg,Always)
          go = .FALSE.
       endif
    else 
       write(errstr,*)  " File: ",trim(file),&
            " does not exist, will create."
       call message(AppMsg,Always)       
    end if
    
    return

  end subroutine gonogo
  !
  !
  !
  subroutine getkeys(in_S,nRows,expo,livetime,schip)

    type(DataSetT) :: in_S
    type(TableT) :: in_T
    character(len=1), intent(in) :: schip
    character(len=8) :: livetimN
    integer, intent(out) :: nRows
    integer :: nBlocks, i
    logical :: gotit
    real(kind=single), intent(out) :: expo, livetime ! should be double, maybe???

    livetimN = "LIVETI0" // schip

    if (.not.hasBlock(in_S,'EVENTS')) then
       call error("noEVENTS","Input file does not contain an EVENTS extension.")
    else
       in_T = table(in_S,"EVENTS")
       nRows = numberOfRows(in_T)
    endif
    call release(in_T)
    !
    nBlocks = numberOfBlocks(in_S)
    gotit = .FALSE.
    do i=0,nBlocks-1
       in_T = table(in_S,i)
       if (hasAttribute(in_T,'EXPOSURE')) then
          expo = real32attribute(in_T,'EXPOSURE')
          gotit = .TRUE.
       end if
       call release(in_T)
    enddo
    if (.not. gotit) then
       call warning("noEXPOSURE","Input file does not contain an EXPOSURE attribute")
    endif
    !
    gotit = .FALSE.
    do i=0,nBlocks-1
       in_T = table(in_S,i)
       if (hasAttribute(in_T,trim(adjustl(livetimN)))) then
          livetime = real32attribute(in_T,trim(adjustl(livetimN)))
          gotit = .TRUE.
       end if
       call release(in_T)
    enddo
    if (.not. gotit) then
       call error("noEXPOSURE","Input file does not contain an LIVETInn attribute")
    endif

    return

  end subroutine getkeys
    
end module pnspectra_mod
